//----------------------------------------------------------------------------
// Copyright (C) 2003  Rafael H. Bordini, Jomi F. Hubner, et al.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// To contact the authors:
// http://www.inf.ufrgs.br/~bordini
// http://www.das.ufsc.br/~jomi
//
//----------------------------------------------------------------------------

package jacamo.util;


import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;

public class ConfigGUI {
    
    protected JTextField jacamoTF;
    protected JTextField javaTF;
    protected JCheckBox  alsoForJasonCB;

    protected static Config userProperties = Config.get();

    public static void main(String[] args) {
        new ConfigGUI().run();
    }
    
    protected static ConfigGUI getNewInstance() {
        return new ConfigGUI();
    }
    
    public void run() {
        userProperties.fix();
        userProperties.remove(Config.MOISE_JAR);
        userProperties.remove(Config.JASON_JAR);
        userProperties.remove(Config.JADE_JAR);
        userProperties.remove(Config.ANT_LIB);
        userProperties.put(Config.SHOW_ANNOTS, "false");
        userProperties.fix();
        final ConfigGUI jid = getNewInstance();
        JFrame f = new JFrame(jid.getWindowTitle());
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        JPanel pBt = new JPanel(new FlowLayout());
        jid.alsoForJasonCB = new JCheckBox("save this configuration also for Jason");
        jid.alsoForJasonCB.setSelected(true);
        pBt.add(jid.alsoForJasonCB);   
       
        JButton bQuit = new JButton("Exit without saving");
        bQuit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                System.exit(0);
            }
        });
        pBt.add(bQuit);

        JButton bSave = new JButton("Save configuration and Exit");
        bSave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                jid.save();
                System.exit(0);
            }
        });
        pBt.add(bSave);

        JPanel p = new JPanel(new BorderLayout());
        p.add(BorderLayout.CENTER, jid.getJasonConfigPanel());
        p.add(BorderLayout.SOUTH, pBt);
        
        f.getContentPane().add(p);
        f.pack();
        f.setVisible(true);
    }
    
    protected String getWindowTitle() {
        return "JaCaMo Configuration -- "+userProperties.getProperty("version");
    }

    public JPanel getJasonConfigPanel() {    
        JPanel pop = new JPanel();
        pop.setLayout(new BoxLayout(pop, BoxLayout.Y_AXIS));

        // jacamo home
        jacamoTF = new JTextField(35);
        JPanel jacamoHomePanel = new JPanel(new GridLayout(0,1));
        jacamoHomePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
                .createEtchedBorder(), "JaCaMo", TitledBorder.LEFT, TitledBorder.TOP));
        JPanel jacamoJarPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        jacamoJarPanel.add(new JLabel("jacamo.jar location"));
        jacamoJarPanel.add(jacamoTF);
        jacamoJarPanel.add(createBrowseButton("jacamo.jar", jacamoTF));
        jacamoHomePanel.add(jacamoJarPanel);

        pop.add(jacamoHomePanel);
        

        // java home
        JPanel javaHomePanel = new JPanel();
        javaHomePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
                .createEtchedBorder(), "Java Home", TitledBorder.LEFT, TitledBorder.TOP));
        javaHomePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        javaHomePanel.add(new JLabel("Directory"));
        javaTF = new JTextField(35);
        javaHomePanel.add(javaTF);
        JButton setJava = new JButton("Browse");
        setJava.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                try {
                    JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
                     chooser.setDialogTitle("Select the Java JDK home directory");
                    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                    if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
                        String javaHome = (new File(chooser.getSelectedFile().getPath())).getCanonicalPath();
                        if (Config.checkJavaHomePath(javaHome)) {
                            javaTF.setText(javaHome);
                        } else {
                            JOptionPane.showMessageDialog(null, "The selected JDK home directory doesn't have the bin/javac file!");
                        }
                    }
                } catch (Exception e) {}
            }
        });
        javaHomePanel.add(setJava);
        pop.add(javaHomePanel);
        
        jacamoTF.setText(userProperties.getProperty(Config.JACAMO_JAR));
        javaTF.setText(userProperties.getJavaHome());
        
        return pop;
    }

    protected JButton createBrowseButton(final String jarfile, final JTextField tf) {
        JButton bt = new JButton("Browse");
        bt.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                try {
                    JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
                    chooser.setDialogTitle("Select the "+jarfile+" file");
                    chooser.setFileFilter(new JarFileFilter(jarfile, "The "+jarfile+" file"));
                    //chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                    if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
                        String selJar = (new File(chooser.getSelectedFile().getPath())).getCanonicalPath();
                        if (Config.checkJar(selJar)) {
                            tf.setText(selJar);
                        } else {
                            JOptionPane.showMessageDialog(null, "The selected "+jarfile+" file was not ok!");
                        }
                    }
                } catch (Exception e) {}
            }
        });
        return bt;
    }

    public void save() {
        if (Config.checkJar(jacamoTF.getText())) {
            userProperties.put(Config.JACAMO_JAR, jacamoTF.getText().trim());
        }

        if (Config.checkJavaHomePath(javaTF.getText()) || Config.checkJREHomePath(javaTF.getText())) {
            userProperties.setJavaHome(javaTF.getText().trim());
        }

        userProperties.store();
        
        // also store Jason conf if not installed
        File jasonConf = jason.jeditplugin.Config.get().getUserConfFile();
        if (!jasonConf.exists() || alsoForJasonCB.isSelected()) {
            userProperties.store(jasonConf);
        }
    }

    class JarFileFilter extends FileFilter {
        String jar,ds;
        public JarFileFilter(String jar, String ds) {
            this.jar = jar;
            this.ds  = ds;
        }
        public boolean accept(File f) {
            if (f.getName().endsWith(jar) || f.isDirectory()) {
                return true;
            } else {
                return false;
            }
        }
        
        public String getDescription() {
            return ds;
        }
    }
}
