package mw;

import cartago.*;

public class CityMap implements SituatedWorkspaceMap, java.io.Serializable {
	
	private GeoPoint center;
	
	public CityMap() {}
	
	public void init(double lat, double lon) throws Exception {
		this.center = new GeoPoint(lat,lon);
	}
	
	public double getDistance(SituatedWorkspacePoint p0, SituatedWorkspacePoint p1) {
		return ((CityPoint)p0).getDistanceFrom((CityPoint)p1);
	}

	public SituatedWorkspacePoint getPointAt(SituatedWorkspacePoint from, SituatedWorkspacePoint to, double distance) {
		CityPoint p1 = (CityPoint) from;
		CityPoint p2 = (CityPoint) to;
		double dist = p2.getDistanceFrom(p1);
		double rap = distance / dist;
		double x = p1.getX() + rap*(p2.getX()-p1.getX());
		double y = p1.getY() + rap*(p2.getY()-p1.getY());    		
		return new CityPoint(x,y);
	}

	public CityPoint toCityPoint(GeoPoint p){
		double dy = GeoPoint.earthRadius*(p.getLatitude() - center.getLatitude());
		double dx = GeoPoint.earthRadius*(p.getLongitude() - center.getLongitude())*Math.cos(center.getLatitude());
		return new CityPoint(dx,dy);
	}
	
	public GeoPoint toGeoPoint(CityPoint p){
		try {
			double lat = p.getY()/GeoPoint.earthRadius + center.getLatitude();
			double lon = p.getX()/(GeoPoint.earthRadius*Math.cos(center.getLatitude()))+center.getLongitude();
			return new GeoPoint(lat*180/Math.PI,lon*180/Math.PI);
		} catch (Exception ex){
			return null;
		}
	}

	public GeoPoint getCenter(){
		return center;
	}

}
