package mw;

import mw.*;
import cartago.*;

public class GPSDeviceDriver extends Artifact {

	private ArtifactId controlled;
	private CityMap map;
	private boolean stopped;
	private static final long SYNCH_TIME = 500;
	
	@OPERATION void init(double lat, double lon) throws Exception {
		map = new CityMap();
		map.init(lat, lon);
		stopped = false;
	}

	@OPERATION void bindTo(ArtifactId controlled) throws Exception {
		this.controlled = controlled;
		CityPoint point = map.toCityPoint(new GeoPoint(44.13964, 12.24250));
		execLinkedOp(controlled,"__init", point);
		execInternalOp("keepSynch");
	}
	
	@OPERATION void moveTo(double lat, double lon) throws Exception {
		CityPoint point = map.toCityPoint(new GeoPoint(lat, lon));
		execLinkedOp(controlled,"__sync", point);
	}
	
	@INTERNAL_OPERATION void keepSynch() throws Exception {
		while (!stopped){
			execLinkedOp(controlled,"__syncNoChanges");
			await_time(SYNCH_TIME);
		}
	}
	
	
}
