package mw;
import javax.swing.*;

import mw.*;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Iterator;

import cartago.*;
import cartago.tools.*;

public class MapGUI extends GUIArtifact {

	private MapFrame frame;
	private OSMService mapService;
	private ArrayList<CityPOI> agentsPos;
	private ArrayList<CityPOI> artifactsPos;
	private CityMap map;
	private double lat,lon;
	private int zoom,w,h;
	
	public void init(String title, double lat, double lon, int zoom, int w, int h) throws Exception {
		init();
		this.lat = lat;
		this.lon = lon;
		this.zoom = zoom;
		this.w = w;
		this.h = h;
		try {
			mapService = new OSMService("Fmjtd%7Cluu8216r29%2Cbg%3Do5-9422g0");
			Image image = mapService.getMap(new GeoPoint(lat,lon), w, h, zoom, new GeoPOI[]{} );
			map = new CityMap();
			map.init(lat,lon);
			agentsPos = new ArrayList<CityPOI>();
			artifactsPos = new ArrayList<CityPOI>();
			// log("got image "+image);
			frame = new MapFrame(title,image,w,h);			
			defineObsProperty("pointer",-1,-1,-1,-1);
			linkMouseEventToOp(frame.getPanel(), "mouseClicked", "mouseClicked");
			
			frame.setVisible(true);		
		} catch (Exception ex){
			ex.printStackTrace();
		}
	}

	@OPERATION void mouseClicked(MouseEvent ev){
		double cx = (ev.getX() - w*0.5)*0.42222;
		double cy = (h*0.5-ev.getY())*0.4182;
		GeoPoint p = map.toGeoPoint(new CityPoint(cx,cy));
		updateObsProperty("pointer",cx,cy,p.getLatitudeDeg(),p.getLongitudeDeg());
		// log("> "+cx+","+cy+" - "+p);
	}
	
	@OPERATION void reloadImage(){
		ArrayList<GeoPOI> points = new ArrayList<GeoPOI>();
		for (CityPOI c: agentsPos){
			points.add(new GeoPOI(c.getLabel(),map.toGeoPoint(c.getPos())));
		}
		for (CityPOI c: artifactsPos){
			points.add(new GeoPOI(c.getLabel(),map.toGeoPoint(c.getPos())));
		}
		GeoPOI[] list = new GeoPOI[points.size()];
		int index = 0;
		for (GeoPOI p: points){
			list[index++] = p;
 		}
		try {
			Image image = mapService.getMap(new GeoPoint(lat,lon), w, h, zoom, list );
			frame.updateImage(image);
		} catch (Exception ex){
			ex.printStackTrace();
		}
	}
	
	@OPERATION void updateAgentPosInMap(String label, double x, double y){
		for (CityPOI p: agentsPos){
			if (p.getLabel().equals(label)){
				p.updatePos(new CityPoint(x,y));
				frame.updateAgentsPos(agentsPos);
				return;
			}
		}
		agentsPos.add(new CityPOI(label, new CityPoint(x,y)));
		frame.updateAgentsPos(agentsPos);
	}
	
	@OPERATION void removeAgentFromMap(String label){
		Iterator<CityPOI> it = agentsPos.iterator();
		while (it.hasNext()){
			if (it.next().getLabel().equals(label)){
				it.remove();
				frame.updateAgentsPos(agentsPos);
				return;
			}
		}
	}

	@OPERATION void updateArtifactPosInMap(String label, double x, double y){
		for (CityPOI p: artifactsPos){
			if (p.getLabel().equals(label)){
				p.updatePos(new CityPoint(x,y));
				frame.updateArtifactsPos(artifactsPos);
				return;
			}
		}
		artifactsPos.add(new CityPOI(label, new CityPoint(x,y)));
		frame.updateArtifactsPos(artifactsPos);
	}
	
	@OPERATION void removeArtifactFromMap(String label){
		Iterator<CityPOI> it = artifactsPos.iterator();
		while (it.hasNext()){
			if (it.next().getLabel().equals(label)){
				it.remove();
				frame.updateArtifactsPos(agentsPos);
				return;
			}
		}
	}

	class MapPanel extends JPanel {
		MapFrame frame;
		int w,h;
		
		public MapPanel(MapFrame frame, int w, int h){
			this.frame = frame;
			setSize(w,h);
			this.w = w;
			this.h = h;
			setPreferredSize(new Dimension(w,h));
		}
		
		@Override
		public void paintComponent(Graphics g){
			Graphics2D g2d = (Graphics2D) g;
			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, // Anti-alias!
				        RenderingHints.VALUE_ANTIALIAS_ON);			
			g.drawImage(frame.getImage(),0,0,null);
			Stroke stroke = new BasicStroke(4f);
			g2d.setStroke(stroke);
			g.setColor(Color.BLUE);
			for (CityPOI p: frame.getArtifactsPos()){
				int px = (int)(w/2 + p.getPos().getX()*2.405482);
				int py = (int)(h/2 - p.getPos().getY()*2.405482);
				g.drawRect(px-8,py-8,16,16);
			}
			g.setColor(Color.RED);
			for (CityPOI p: frame.getAgentsPos()){
				int px = (int)(w/2 + p.getPos().getX()*2.405482);
				int py = (int)(h/2 - p.getPos().getY()*2.405482);
				g.drawOval(px-8,py-8,16,16);
			}
		}
	}

	class MapFrame extends JFrame {		
		private Image image;
		private ArrayList<CityPOI> agentsPos = new ArrayList<CityPOI> ();
		private ArrayList<CityPOI> artifactsPos = new ArrayList<CityPOI>  ();
		private MapPanel panel;
		
		public MapFrame(String title, Image image, int w, int h){
			this.image = image;
			panel = new MapPanel(this, w,h); 
			getContentPane().add(panel); 
			setSize(w,h);
			setPreferredSize(new Dimension(w,h));
			setTitle(title);
			pack();	
		}
		
		public MapPanel getPanel(){
			return panel;
		}
		
		public Image getImage(){
			return image;
		}
		
		public void updateImage(Image image){
			this.image = image;
			repaint();
		}
		
		public void updateAgentsPos(ArrayList<CityPOI> agentsPos){
			this.agentsPos = (ArrayList<CityPOI>) agentsPos.clone();
			repaint();
		}

		public void updateArtifactsPos(ArrayList<CityPOI> artifactsPos){
			this.artifactsPos = (ArrayList<CityPOI>) artifactsPos.clone();		
			repaint();
		}
		
		public ArrayList<CityPOI> getArtifactsPos(){
			return artifactsPos;
		}

		public ArrayList<CityPOI> getAgentsPos(){
			return agentsPos;
		}
	}
}
